@@ -2,7 +2,7 @@
  * log.c	Functions in the library call radlib_log() which
  *		sets a global error string "char *librad_errstr".
  *
- * Version:	$Id: log.c,v 1.7 2003/09/12 19:25:29 phampson Exp $
+ * Version:	$Id: log.c,v 1.7.2.1 2005/09/04 16:54:13 nbk Exp $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
@@ -21,7 +21,7 @@
  * Copyright 2000  The FreeRADIUS server project
  */
 
-static const char rcsid[] = "$Id: log.c,v 1.7 2003/09/12 19:25:29 phampson Exp $";
+static const char rcsid[] = "$Id: log.c,v 1.7.2.1 2005/09/04 16:54:13 nbk Exp $";
 
 #include "autoconf.h"
 
@@ -47,15 +47,7 @@ void librad_log(const char *fmt, ...)
 	char my_errstr[sizeof(librad_errstr)];
 
 	va_start(ap, fmt);
-
-#ifdef HAVE_VSNPRINTF
 	vsnprintf(my_errstr, sizeof(my_errstr), fmt, ap);
-#else
-	vsprintf(my_errstr, fmt, ap);
-	if (strlen(my_errstr) >= sizeof(my_errstr))
-		/* What can we do .. */
-		_exit(42);
-#endif
 	strcpy(librad_errstr, my_errstr);
 	va_end(ap);
 }
