@@ -1,7 +1,7 @@
 /*
  * misc.c	Various miscellaneous functions.
  *
- * Version:	$Id: misc.c,v 1.41.2.3 2005/08/24 14:37:51 nbk Exp $
+ * Version:	$Id: misc.c,v 1.41.2.3.2.1 2005/12/12 23:28:29 aland Exp $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
@@ -21,7 +21,7 @@
  */
 
 static const char rcsid[] =
-"$Id: misc.c,v 1.41.2.3 2005/08/24 14:37:51 nbk Exp $";
+"$Id: misc.c,v 1.41.2.3.2.1 2005/12/12 23:28:29 aland Exp $";
 
 #include	"autoconf.h"
 
@@ -426,3 +426,46 @@ int ipv6_addr(const char *ip6_str, void *ip6addr)
 #endif
 	return 0;
 }
+
+static const char *hextab = "0123456789abcdef";
+
+/*
+ *	hex2bin
+ *
+ *	We allow: hex == bin
+ */
+int lrad_hex2bin(const char *hex, uint8_t *bin, int len)
+{
+	int i;
+	char *c1, *c2;
+
+	for (i = 0; i < len; i++) {
+		if(!(c1 = memchr(hextab, tolower((int) hex[i << 1]), 16)) ||
+		   !(c2 = memchr(hextab, tolower((int) hex[(i << 1) + 1]), 16)))
+			break;
+                 bin[i] = ((c1-hextab)<<4) + (c2-hextab);
+	}
+
+	return i;
+}
+
+
+/*
+ *	bin2hex
+ *
+ *	If the output buffer isn't long enough, we have a buffer overflow.
+ */
+void lrad_bin2hex(const uint8_t *bin, char *hex, int len)
+{
+	int i;
+
+	for (i = 0; i < len; i++) {
+		hex[0] = hextab[((*bin) >> 4) & 0x0f];
+		hex[1] = hextab[*bin & 0x0f];
+		hex += 2;
+		bin++;
+	}
+	*hex = '\0';
+	return;
+}
+
