@@ -1,7 +1,7 @@
 /*
  * print.c	Routines to print stuff.
  *
- * Version:	$Id: print.c,v 1.26.2.1 2005/04/21 22:06:06 aland Exp $
+ * Version:	$Id: print.c,v 1.26.2.1.2.1 2005/12/12 10:52:01 nbk Exp $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
@@ -20,7 +20,7 @@
  * Copyright 2000  The FreeRADIUS server project
  */
 
-static const char rcsid[] = "$Id: print.c,v 1.26.2.1 2005/04/21 22:06:06 aland Exp $";
+static const char rcsid[] = "$Id: print.c,v 1.26.2.1.2.1 2005/12/12 10:52:01 nbk Exp $";
 
 #include	"autoconf.h"
 
@@ -170,12 +170,7 @@ int vp_prints_value(char * out, int outlen, VALUE_PAIR *vp, int delimitst)
 			a = buf;
 			break;
 		case PW_TYPE_IPADDR:
-			if (vp->strvalue[0])
-				a = (char *)vp->strvalue;
-			else
-				a = ip_hostname((char *)vp->strvalue,
-						sizeof(vp->strvalue),
-						vp->lvalue);
+			a = ip_ntoa(buf, vp->lvalue);
 			break;
 		case PW_TYPE_ABINARY:
 #ifdef ASCEND_BINARY
