@@ -5,7 +5,7 @@
  *		write a decent parser. I know how to do that, really :)
  *		miquels@cistron.nl
  *
- * Version:	$Id: conffile.c,v 1.100 2004/04/06 18:00:57 aland Exp $
+ * Version:	$Id: conffile.c,v 1.100.2.1 2005/10/03 17:18:59 nbk Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@
 #include "modules.h"
 
 static const char rcsid[] =
-"$Id: conffile.c,v 1.100 2004/04/06 18:00:57 aland Exp $";
+"$Id: conffile.c,v 1.100.2.1 2005/10/03 17:18:59 nbk Exp $";
 
 #define xstrdup strdup
 
@@ -669,7 +669,7 @@ static CONF_SECTION *cf_section_read(const char *cf, int *lineno, FILE *fp,
 
 			DEBUG2( "Config:   including file: %s", value );
 
-			if ((is = conf_read(cf, *lineno, value, parent)) == NULL) {
+			if ((is = conf_read(cf, *lineno, value, cs)) == NULL) {
 				cf_section_free(&cs);
 				return NULL;
 			}
