@@ -1,7 +1,7 @@
 /*
  * exec.c	Execute external programs.
  *
- * Version:	$Id: exec.c,v 1.42 2004/02/26 19:04:22 aland Exp $
+ * Version:	$Id: exec.c,v 1.42.2.1 2005/08/26 19:17:45 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
  * Copyright 2000  The FreeRADIUS server project
  * Copyright 2000  Michael J. Hartwick <hartwick@hartwick.com>
  */
-static const char rcsid[] = "$Id: exec.c,v 1.42 2004/02/26 19:04:22 aland Exp $";
+static const char rcsid[] = "$Id: exec.c,v 1.42.2.1 2005/08/26 19:17:45 aland Exp $";
 
 #include "autoconf.h"
 
@@ -228,6 +228,11 @@ int radius_exec_program(const char *cmd, REQUEST *request,
 			vp_prints_value(buffer+n, sizeof(buffer) - n, vp, 1);
 
 			envp[envlen++] = strdup(buffer);
+
+			/*
+			 *	Don't add too many attributes.
+			 */
+			if (envlen == (MAX_ENVP - 1)) break;
 		}
 		envp[envlen] = NULL;
 		execve(argv[0], argv, envp);
