@@ -1,7 +1,7 @@
 /*
  * modcall.c
  *
- * Version:	$Id: modcall.c,v 1.22.2.1 2005/02/09 18:19:00 aland Exp $
+ * Version:	$Id: modcall.c,v 1.22.2.1.2.3 2005/12/15 00:44:34 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -46,13 +46,19 @@ static modcallable *do_compile_modgroup(int, CONF_SECTION *, const char *,
  * explanation of what they are all about, see ../../doc/README.failover */
 struct modcallable {
 	struct modcallable *next;
-	int actions[RLM_MODULE_NUMCODES];
 	const char *name;
-	enum { MOD_SINGLE, MOD_GROUP } type;
+	int actions[RLM_MODULE_NUMCODES];
+	enum { MOD_SINGLE, MOD_GROUP, MOD_LOAD_BALANCE, MOD_REDUNDANT_LOAD_BALANCE  } type;
 };
 
+#define GROUPTYPE_SIMPLE	0
+#define GROUPTYPE_REDUNDANT	1
+#define GROUPTYPE_APPEND	2
+#define GROUPTYPE_COUNT		3
+
 typedef struct {
 	modcallable mc;
+	int grouptype;
 	modcallable *children;
 } modgroup;
 
@@ -61,6 +67,14 @@ typedef struct {
 	module_instance_t *modinst;
 } modsingle;
 
+
+static const LRAD_NAME_NUMBER grouptype_table[] = {
+	{ "", GROUPTYPE_SIMPLE },
+	{ "redundant ", GROUPTYPE_REDUNDANT },
+	{ "append ", GROUPTYPE_APPEND },
+	{ NULL, -1 }
+};
+
 /* Simple conversions: modsingle and modgroup are subclasses of modcallable,
  * so we often want to go back and forth between them. */
 static modsingle *mod_callabletosingle(modcallable *p)
@@ -70,7 +84,9 @@ static modsingle *mod_callabletosingle(modcallable *p)
 }
 static modgroup *mod_callabletogroup(modcallable *p)
 {
-	rad_assert(p->type==MOD_GROUP);
+	rad_assert((p->type==MOD_GROUP) ||
+		   (p->type==MOD_LOAD_BALANCE) ||
+		   (p->type==MOD_REDUNDANT_LOAD_BALANCE));
 	return (modgroup *)p;
 }
 static modcallable *mod_singletocallable(modsingle *p)
@@ -89,6 +105,8 @@ static void add_child(modgroup *g, modcallable *c)
 	modcallable *node = *head;
 	modcallable **last = head;
 
+	if (!c) return;
+
 	while (node) {
 		last = &node->next;
 		node = node->next;
@@ -100,7 +118,7 @@ static void add_child(modgroup *g, modcallable *c)
 
 /* Here's where we recognize all of our keywords: first the rcodes, then the
  * actions */
-static LRAD_NAME_NUMBER rcode_table[] = {
+static const LRAD_NAME_NUMBER rcode_table[] = {
 	{ "reject",     RLM_MODULE_REJECT       },
 	{ "fail",       RLM_MODULE_FAIL         },
 	{ "ok",         RLM_MODULE_OK           },
@@ -113,47 +131,46 @@ static LRAD_NAME_NUMBER rcode_table[] = {
 	{ NULL, 0 }
 };
 
-static int str2rcode(const char *s, const char *filename, int lineno)
-{
-	int rcode;
 
-	rcode = lrad_str2int(rcode_table, s, -1);
+/*
+ *	Compile action && rcode for later use.
+ */
+static int compile_action(modcallable *c, const char *attr, const char *value,
+			  const char *filename, int lineno)
+{
+	int rcode, action;
+
+	rcode = lrad_str2int(rcode_table, attr, -1);
 	if (rcode < 0) {
 		radlog(L_ERR|L_CONS,
 		       "%s[%d] Unknown module rcode '%s'.\n",
-		       filename, lineno, s);
-		exit(1);
+		       filename, lineno, attr);
+		return 0;
 	}
 
-	return rcode;
-}
+	if (!strcasecmp(value, "return"))
+		action = MOD_ACTION_RETURN;
 
-static int str2action(const char *s, const char *filename, int lineno)
-{
-	if(!strcasecmp(s, "return"))
-		return MOD_ACTION_RETURN;
-	else if(!strcasecmp(s, "reject"))
-		return MOD_ACTION_REJECT;
-	else if (strspn(s, "0123456789")==strlen(s)) {
-		int rcode = atoi(s);
+	else if (!strcasecmp(value, "reject"))
+		action = MOD_ACTION_REJECT;
 
+	else if (strspn(value, "0123456789")==strlen(value)) {
+		action = atoi(value);
+		
 		/*
 		 *	Don't allow priority zero, for future use.
 		 */
-		if (rcode == 0) {
-			radlog(L_ERR|L_CONS,
-			       "%s[%d] Invalid action '%s'.\n",
-			       filename, lineno, s);
-			exit(1);
-		}
-		return rcode;
+		if (action == 0) return 0;
 	} else {
 		radlog(L_ERR|L_CONS,
-				"%s[%d] Unknown action '%s'.\n",
-				filename, lineno, s);
-		exit(1);
+		       "%s[%d] Unknown action '%s'.\n",
+		       filename, lineno, value);
+		return 0;
 	}
-	return -1;
+
+	c->actions[rcode] = action;
+
+	return 1;
 }
 
 #if 0
@@ -226,11 +243,72 @@ static int call_modsingle(int component, modsingle *sp, REQUEST *request,
 	return myresult;
 }
 
+
+/*
+ *	Helper function for call_modgroup, and call_modredundantloadbalance
+ *
+ *	Returns 0 for "stop", and "1" for continue.
+ */
+static int call_one(int component, modcallable *p, REQUEST *request,
+		    int *priority, int *result)
+{
+	int r;
+
+#ifdef RAD_REQUEST_OPTION_STOP_NOW
+	/*
+	 *	A module has taken too long to process the request,
+	 *	and we've been told to stop processing it.
+	 */
+	if (request->options & RAD_REQUEST_OPTION_STOP_NOW) {
+		*result = RLM_MODULE_FAIL;
+		return 0;
+	}
+#endif
+	
+	/* Call this child by recursing into modcall */
+	r = modcall(component, p, request);
+	
+#if 0
+	DEBUG2("%s: action for %s is %s",
+	       comp2str[component], lrad_int2str(rcode_table, r, "??"),
+	       action2str(p->actions[r]));
+#endif
+	
+	/*
+	 * 	Find an action to go with the child's result. If it is
+	 * 	"return", break out of the loop so the rest of the
+	 * 	children in the list will be skipped.
+	 */
+	if (p->actions[r] == MOD_ACTION_RETURN) {
+		*result = r;
+		return 0;
+	}
+	
+	/* If "reject" break out of the loop and return reject */
+	if (p->actions[r] == MOD_ACTION_REJECT) {
+		*result = RLM_MODULE_REJECT;
+		return 0;
+	}
+	
+	/*
+	 *	Otherwise, the action is a number, the preference
+	 *	level of this return code. If no higher preference has
+	 *	been seen yet, remember this one
+	 . */
+	if (p->actions[r] >= *priority) {
+		*result = r;
+		*priority = p->actions[r];
+	}
+	
+	return 1;
+}
+
+
 static int call_modgroup(int component, modgroup *g, REQUEST *request,
-		int default_result)
+			 int default_result)
 {
 	int myresult = default_result;
-	int myresultpref;
+	int priority = 0;	/* default result has lowest priority  */
 	modcallable *p;
 
 	/*
@@ -241,48 +319,147 @@ static int call_modgroup(int component, modgroup *g, REQUEST *request,
 		return default_result;
 	}
 
-	/* Assign the lowest possible preference to the default return code */
-	myresultpref = 0;
-
 	/* Loop over the children */
-	for(p = g->children; p; p = p->next) {
-		int r = RLM_MODULE_FAIL;
-
-		/* Call this child by recursing into modcall */
-		r = modcall(component, p, request);
-
-#if 0
-		DEBUG2("%s: action for %s is %s",
-			comp2str[component], lrad_int2str(rcode_table, r, "??"),
-			action2str(p->actions[r]));
-#endif
-
-		/* Find an action to go with the child's result. If "return",
-		 * break out of the loop so the rest of the children in the
-		 * list will be skipped. */
-		if(p->actions[r] == MOD_ACTION_RETURN) {
-			myresult = r;
+	for (p = g->children; p; p = p->next) {
+		if (!call_one(component, p, request, &priority, &myresult)) {
 			break;
 		}
-
-		/* If "reject" break out of the loop and return reject */
-		if (p->actions[r] == MOD_ACTION_REJECT) {
-			myresult = RLM_MODULE_REJECT;
-			break;
-		}
-
-		/* Otherwise, the action is a number, the preference level of
-		 * this return code. If no higher preference has been seen
-		 * yet, remember this one. */
-		if(p->actions[r] >= myresultpref) {
-			myresult = r;
-			myresultpref = p->actions[r];
-		}
 	}
 
 	return myresult;
 }
 
+static int call_modloadbalance(int component, modgroup *g, REQUEST *request,
+			       int default_result)
+{
+	int count = 1;
+	modcallable *p, *child = NULL;
+
+	/*
+	 *	Catch people who have issues.
+	 */
+	if (!g->children) {
+		DEBUG2("  WARNING! Asked to process empty load-balance group.  Returning %s.", lrad_int2str(rcode_table, default_result, "??"));
+		return default_result;
+	}
+
+	/*
+	 *	Pick a random child.
+	 */
+
+	/* Loop over the children */
+	for(p = g->children; p; p = p->next) {
+		if (!child) {
+			child = p;
+			count = 1;
+			continue;
+		}
+
+		/*
+		 *	Keep track of how many load balancing servers
+		 *	we've gone through.
+		 */
+		count++;
+
+		/*
+		 *	See the "camel book" for why this works.
+		 *
+		 *	If (rand(0..n) < 1), pick the current realm.
+		 *	We add a scale factor of 65536, to avoid
+		 *	floating point.
+		 */
+		if ((count * (lrad_rand() & 0xffff)) < (uint32_t) 0x10000) {
+			child = p;
+		}
+	}
+	rad_assert(child != NULL);
+
+	/* Call the chosen child by recursing into modcall */
+	return modcall(component, child, request);
+}
+
+
+/*
+ *	For more than 2 modules with redundancy + load balancing
+ *	across all of them, layering the "redundant" and
+ *	"load-balance" groups gets too complicated.  As a result, we
+ *	implement a special function to do this.
+ */
+static int call_modredundantloadbalance(int component, modgroup *g, REQUEST *request,
+					int default_result)
+{
+	int count = 1;
+	int myresult = default_result;
+	int priority = 0;	/* default result has lowest priority  */
+	modcallable *p, *child = NULL;
+
+	/*
+	 *	Catch people who have issues.
+	 */
+	if (!g->children) {
+		DEBUG2("  WARNING! Asked to process empty redundant-load-balance group.  Returning %s.", lrad_int2str(rcode_table, default_result, "??"));
+		return default_result;
+	}
+
+	/*
+	 *	Pick a random child.
+	 */
+
+	/* Loop over the children */
+	for(p = g->children; p; p = p->next) {
+		if (!child) {
+			child = p;
+			count = 1;
+			continue;
+		}
+
+		/*
+		 *	Keep track of how many load balancing servers
+		 *	we've gone through.
+		 */
+		count++;
+
+		/*
+		 *	See the "camel book" for why this works.
+		 *
+		 *	If (rand(0..n) < 1), pick the current realm.
+		 *	We add a scale factor of 65536, to avoid
+		 *	floating point.
+		 */
+		if ((count * (lrad_rand() & 0xffff)) < (uint32_t) 0x10000) {
+			child = p;
+		}
+	}
+	rad_assert(child != NULL);
+
+	/*
+	 *	Call the chosen child, with fail-over to the next one
+	 *	if it is down.
+	 */
+	p = child;
+	do {
+		/*
+		 *	Call the chosen entry.  If we're done, then
+		 *	stop.
+		 */
+		if (!call_one(component, p, request, &priority, &myresult)) {
+			break;
+		}
+		
+		/*
+		 *	Go to the next one, and wrap around to the beginning if
+		 *	we reach the end.
+		 */
+		p = p->next;
+		if (!p) p = g->children;
+	} while (p != child);
+
+	/*
+	 *	And return whatever was decided.
+	 */
+	return myresult;
+}
+
 int modcall(int component, modcallable *c, REQUEST *request)
 {
 	int myresult;
@@ -326,27 +503,77 @@ int modcall(int component, modcallable *c, REQUEST *request)
 		return myresult;
 	}
 
-	if(c->type==MOD_GROUP) {
-		modgroup *g = mod_callabletogroup(c);
+	switch (c->type) {
+	case MOD_LOAD_BALANCE:
+		{
+			modgroup *g = mod_callabletogroup(c);
+			
+			DEBUG2("modcall: entering load-balance group %s for request %d",
+			       c->name, request->number);
+			
+			myresult = call_modloadbalance(component, g, request,
+						       myresult);
+			
+			DEBUG2("modcall: load-balance group %s returns %s for request %d",
+			       c->name,
+			       lrad_int2str(rcode_table, myresult, "??"),
+			       request->number);
+		}
+		break;
+		
+	case MOD_REDUNDANT_LOAD_BALANCE:
+		{
+			modgroup *g = mod_callabletogroup(c);
+			
+			DEBUG2("modcall: entering redundant-load-balance group %s for request %d",
+			       c->name, request->number);
+			
+			myresult = call_modredundantloadbalance(component, g, request,
+								myresult);
+			
+			DEBUG2("modcall: redundant-load-balance group %s returns %s for request %d",
+			       c->name,
+			       lrad_int2str(rcode_table, myresult, "??"),
+			       request->number);
+		}
+		break;
+		
+	case MOD_GROUP:
+		{
+			modgroup *g = mod_callabletogroup(c);
+			
+			DEBUG2("modcall: entering group %s%s for request %d",
+			       lrad_int2str(grouptype_table, g->grouptype, ""),
+			       c->name, request->number);
+			
+			myresult = call_modgroup(component, g, request,
+						 myresult);
+			
+			DEBUG2("modcall: leaving group %s%s (returns %s) for request %d",
+			       lrad_int2str(grouptype_table, g->grouptype, ""),
+			       c->name,
+			       lrad_int2str(rcode_table, myresult, "??"),
+			       request->number);
+		}
+		break;
+		
+	case MOD_SINGLE:
+		{
+			modsingle *sp = mod_callabletosingle(c);
+			
+			myresult = call_modsingle(component, sp, request,
+						  myresult);
+			
+			DEBUG2("  modcall[%s]: module \"%s\" returns %s for request %d",
+			       comp2str[component], c->name,
+			       lrad_int2str(rcode_table, myresult, "??"),
+			       request->number);
+		}
+		break;
 
-		DEBUG2("modcall: entering group %s for request %d",
-		       c->name, request->number);
-
-		myresult = call_modgroup(component, g, request, myresult);
-
-		DEBUG2("modcall: group %s returns %s for request %d",
-		        c->name,
-		       lrad_int2str(rcode_table, myresult, "??"),
-		       request->number);
-	} else {
-		modsingle *sp = mod_callabletosingle(c);
-
-		myresult = call_modsingle(component, sp, request, myresult);
-
-		DEBUG2("  modcall[%s]: module \"%s\" returns %s for request %d",
-		       comp2str[component], c->name,
-		       lrad_int2str(rcode_table, myresult, "??"),
-		       request->number);
+	default:
+		radlog(L_ERR, "Internal error processing module entry");
+		break;
 	}
 
 	return myresult;
@@ -393,15 +620,10 @@ static void dump_tree(int comp UNUSED, modcallable *c UNUSED)
 }
 #endif
 
-#define GROUPTYPE_SIMPLEGROUP 0
-#define GROUPTYPE_REDUNDANT 1
-#define GROUPTYPE_APPEND 2
-#define GROUPTYPE_COUNT 3
-
 /* These are the default actions. For each component, the group{} block
  * behaves like the code from the old module_*() function. redundant{} and
  * append{} are based on my guesses of what they will be used for. --Pac. */
-static int
+static const int
 defaultactions[RLM_COMPONENT_COUNT][GROUPTYPE_COUNT][RLM_MODULE_NUMCODES] =
 {
 	/* authenticate */
@@ -718,49 +940,10 @@ defaultactions[RLM_COMPONENT_COUNT][GROUPTYPE_COUNT][RLM_MODULE_NUMCODES] =
 	}
 };
 
-/* Bail out if the module in question does not supply the wanted component */
-static void sanity_check(int component, module_instance_t *inst,
-			 const char *filename)
-{
-	if (!inst->entry->module->methods[component]) {
-		radlog(L_ERR|L_CONS,
-				"%s: \"%s\" modules aren't allowed in '%s' sections -- they have no such method.",
-				filename, inst->entry->module->name,
-				component_names[component]);
-		exit(1);
-	}
-}
-
-/* Parse a CONF_SECTION containing only result=action pairs */
-static void override_actions(modcallable *c, CONF_SECTION *cs,
-		const char *filename)
-{
-	CONF_ITEM *ci;
-	CONF_PAIR *cp;
-	const char *attr, *value;
-	int lineno, rcode, action;
-
-	for(ci=cf_item_find_next(cs, NULL); ci != NULL; ci=cf_item_find_next(cs, ci)) {
-		if(cf_item_is_section(ci)) {
-			radlog(L_ERR|L_CONS,
-					"%s[%d] Subsection of module instance call "
-					"not allowed\n", filename,
-					cf_section_lineno(cf_itemtosection(ci)));
-			exit(1);
-		}
-		cp = cf_itemtopair(ci);
-		attr = cf_pair_attr(cp);
-		value = cf_pair_value(cp);
-		lineno = cf_pair_lineno(cp);
-		rcode = str2rcode(attr, filename, lineno);
-		action = str2action(value, filename, lineno);
-		c->actions[rcode] = action;
-	}
-}
 
 static modcallable *do_compile_modsingle(int component, CONF_ITEM *ci,
-		const char *filename, int grouptype,
-		const char **modname)
+					 const char *filename, int grouptype,
+					 const char **modname)
 {
 	int lineno;
 	const char *modrefname;
@@ -770,25 +953,44 @@ static modcallable *do_compile_modsingle(int component, CONF_ITEM *ci,
 
 	if (cf_item_is_section(ci)) {
 		CONF_SECTION *cs = cf_itemtosection(ci);
+		const char *name2 = cf_section_name2(cs);
 
 		lineno = cf_section_lineno(cs);
 		modrefname = cf_section_name1(cs);
+		if (!name2) name2 = "_UnNamedGroup";
 
-		/* group{}, redundant{}, or append{} may appear where a
-		 * single module instance was expected - in that case, we
-		 * hand it off to compile_modgroup */
+		/*
+		 *	group{}, redundant{}, or append{} may appear
+		 *	where a single module instance was expected.
+		 *	In that case, we hand it off to
+		 *	compile_modgroup
+		 */
 		if (strcmp(modrefname, "group") == 0) {
-			*modname = "UnnamedGroup";
+			*modname = name2;
 			return do_compile_modgroup(component, cs, filename,
-					GROUPTYPE_SIMPLEGROUP, grouptype);
+					GROUPTYPE_SIMPLE, grouptype);
 		} else if (strcmp(modrefname, "redundant") == 0) {
-			*modname = "UnnamedGroup";
+			*modname = name2;
 			return do_compile_modgroup(component, cs, filename,
 					GROUPTYPE_REDUNDANT, grouptype);
 		} else if (strcmp(modrefname, "append") == 0) {
-			*modname = "UnnamedGroup";
+			*modname = name2;
 			return do_compile_modgroup(component, cs, filename,
 					GROUPTYPE_APPEND, grouptype);
+		} else if (strcmp(modrefname, "load-balance") == 0) {
+			*modname = name2;
+			csingle= do_compile_modgroup(component, cs, filename,
+					GROUPTYPE_SIMPLE, grouptype);
+			if (!csingle) return NULL;
+			csingle->type = MOD_LOAD_BALANCE;
+			return csingle;
+		} else if (strcmp(modrefname, "redundant-load-balance") == 0) {
+			*modname = name2;
+			csingle= do_compile_modgroup(component, cs, filename,
+					GROUPTYPE_REDUNDANT, grouptype);
+			if (!csingle) return NULL;
+			csingle->type = MOD_REDUNDANT_LOAD_BALANCE;
+			return csingle;
 		}
 	} else {
 		CONF_PAIR *cp = cf_itemtopair(ci);
@@ -796,69 +998,147 @@ static modcallable *do_compile_modsingle(int component, CONF_ITEM *ci,
 		modrefname = cf_pair_attr(cp);
 	}
 
+	/*
+	 *	FIXME: See if the module is a virtual one.  If so,
+	 *	return that, rather than doing anything here.
+	 */
+	this = find_module_instance(modrefname);
+	if (!this) {
+		*modname = NULL;
+		radlog(L_ERR|L_CONS, "%s[%d] Unknown module \"%s\".", filename,
+		       lineno, modrefname);
+		return NULL;
+	}
+
+	/*
+	 *	We know it's all OK, allocate the structures, and fill
+	 *	them in.
+	 */
 	single = rad_malloc(sizeof(*single));
 	csingle = mod_singletocallable(single);
 	csingle->next = NULL;
-	memcpy(csingle->actions,
-			defaultactions[component][grouptype],
-			sizeof csingle->actions);
+	memcpy(csingle->actions, defaultactions[component][grouptype],
+	       sizeof csingle->actions);
 	rad_assert(modrefname != NULL);
 	csingle->name = modrefname;
 	csingle->type = MOD_SINGLE;
 
+	/*
+	 *	Singles can override the actions, virtual modules cannot.
+	 *
+	 *	FIXME: We may want to re-visit how to do this...
+	 *	maybe a csingle as a ref?
+	 */
 	if (cf_item_is_section(ci)) {
-		/* override default actions with what's in the CONF_SECTION */
-		override_actions(csingle, cf_itemtosection(ci), filename);
+		CONF_SECTION *cs = cf_itemtosection(ci);
+		CONF_PAIR *cp;
+		const char *attr, *value;
+
+		for (ci=cf_item_find_next(cs, NULL);
+		     ci != NULL;
+		     ci=cf_item_find_next(cs, ci)) {
+
+			if (cf_item_is_section(ci)) {
+				radlog(L_ERR|L_CONS,
+				       "%s[%d] Subsection of module instance call "
+				       "not allowed\n", filename,
+				       cf_section_lineno(cf_itemtosection(ci)));
+				modcallable_free(&csingle);
+				return NULL;
+			}
+
+			cp = cf_itemtopair(ci);
+			attr = cf_pair_attr(cp);
+			value = cf_pair_value(cp);
+			lineno = cf_pair_lineno(cp);
+
+			if (!compile_action(csingle, attr, value, filename,
+					    lineno)) {
+				modcallable_free(&csingle);
+				return NULL;
+			}
+		}
 	}
 
-	this = find_module_instance(modrefname);
-	if (this == NULL) {
-		exit(1); /* FIXME */
+	/*
+	 *	Bail out if the module in question does not supply the
+	 *	wanted component
+	 */
+	if (!this->entry->module->methods[component]) {
+		radlog(L_ERR|L_CONS,
+		       "%s: \"%s\" modules aren't allowed in '%s' sections -- they have no such method.",
+		       filename, this->entry->module->name,
+		       component_names[component]);
+		modcallable_free(&csingle);
+		return NULL;
 	}
 
-	sanity_check(component, this, filename);
-
 	single->modinst = this;
 	*modname = this->entry->module->name;
 	return csingle;
 }
 
 modcallable *compile_modsingle(int component, CONF_ITEM *ci,
-		const char *filename, const char **modname)
+			       const char *filename, const char **modname)
 {
 	modcallable *ret = do_compile_modsingle(component, ci, filename,
-		GROUPTYPE_SIMPLEGROUP,
-		modname);
+						GROUPTYPE_SIMPLE,
+						modname);
 	dump_tree(component, ret);
 	return ret;
 }
 
 static modcallable *do_compile_modgroup(int component, CONF_SECTION *cs,
-		const char *filename, int grouptype,
-		int parentgrouptype)
+					const char *filename, int grouptype,
+					int parentgrouptype)
 {
 	modgroup *g;
 	modcallable *c;
 	CONF_ITEM *ci;
 
 	g = rad_malloc(sizeof *g);
+	g->grouptype = grouptype;
 
 	c = mod_grouptocallable(g);
 	c->next = NULL;
 	memcpy(c->actions, defaultactions[component][parentgrouptype],
-		sizeof c->actions);
-	c->name = cf_section_name1(cs);
-	rad_assert(c->name != NULL);
+	       sizeof(c->actions));
+
+	/*
+	 *	Remember the name for printing, etc.
+	 *
+	 *	FIXME: We may also want to put the names into a
+	 *	rbtree, so that groups can reference each other...
+	 */
+	c->name = cf_section_name2(cs);
+	if (!c->name) c->name = "";
 	c->type = MOD_GROUP;
 	g->children = NULL;
 
-	for (ci=cf_item_find_next(cs, NULL); ci != NULL; ci=cf_item_find_next(cs, ci)) {
+	for (ci=cf_item_find_next(cs, NULL);
+	     ci != NULL;
+	     ci=cf_item_find_next(cs, ci)) {
+
 		if(cf_item_is_section(ci)) {
-			const char *junk;
+			const char *junk = NULL;
 			modcallable *single;
+			int lineno;
+			CONF_SECTION *subcs = cf_itemtosection(ci);
+
+			lineno = cf_section_lineno(subcs);
+
 			single = do_compile_modsingle(component, ci, filename,
-					grouptype, &junk);
+						      grouptype, &junk);
+			if (!single) {
+				radlog(L_ERR|L_CONS,
+				       "%s[%d] Failed to parse \"%s\" subsection.\n",
+				       filename, lineno,
+				       cf_section_name1(subcs));
+				modcallable_free(&c);
+				return NULL;
+			}
 			add_child(g, single);
+
 		} else {
 			const char *attr, *value;
 			CONF_PAIR *cp = cf_itemtopair(ci);
@@ -868,27 +1148,41 @@ static modcallable *do_compile_modgroup(int component, CONF_SECTION *cs,
 			value = cf_pair_value(cp);
 			lineno = cf_pair_lineno(cp);
 
-			/* A CONF_PAIR is either a module instance with no
-			 * actions specified... */
-			if(value[0]==0) {
+			/*
+			 *	A CONF_PAIR is either a module
+			 *	instance with no actions
+			 *	specified ...
+			 */
+			if (value[0] == 0) {
 				modcallable *single;
-				const char *junk;
+				const char *junk = NULL;
 
 				single = do_compile_modsingle(component,
 						cf_pairtoitem(cp), filename,
 						grouptype, &junk);
+				if (!single) {
+					radlog(L_ERR|L_CONS,
+					       "%s[%d] Failed to parse \"%s\" entry.\n",
+					       filename, lineno, attr);
+					modcallable_free(&c);
+					return NULL;
+				}
 				add_child(g, single);
-			} else {
-				/* ...or an action to be applied to this
-				 * group. */
-				int rcode, action;
-				rcode = str2rcode(attr, filename, lineno);
-				action = str2action(value, filename, lineno);
 
-				c->actions[rcode] = action;
-			}
+				/*
+				 *	Or a module instance with action.
+				 */
+			} else if (!compile_action(c, attr, value, filename,
+						   lineno)) {
+				modcallable_free(&c);
+				return NULL;
+			} /* else it worked */
 		}
 	}
+
+	/*
+	 *	FIXME: If there are no children, return NULL?
+	 */
 	return mod_grouptocallable(g);
 }
 
@@ -896,26 +1190,29 @@ modcallable *compile_modgroup(int component, CONF_SECTION *cs,
 		const char *filename)
 {
 	modcallable *ret = do_compile_modgroup(component, cs, filename,
-			GROUPTYPE_SIMPLEGROUP,
-			GROUPTYPE_SIMPLEGROUP);
+					       GROUPTYPE_SIMPLE,
+					       GROUPTYPE_SIMPLE);
 	dump_tree(component, ret);
 	return ret;
 }
 
 void add_to_modcallable(modcallable **parent, modcallable *this,
-		int component, char *name)
+			int component, char *name)
 {
 	modgroup *g;
+	
+	rad_assert(this != NULL);
 
 	if (*parent == NULL) {
 		modcallable *c;
 
 		g = rad_malloc(sizeof *g);
+		g->grouptype = GROUPTYPE_SIMPLE;
 		c = mod_grouptocallable(g);
 		c->next = NULL;
 		memcpy(c->actions,
-				defaultactions[component][GROUPTYPE_SIMPLEGROUP],
-				sizeof c->actions);
+		       defaultactions[component][GROUPTYPE_SIMPLE],
+		       sizeof(c->actions));
 		rad_assert(name != NULL);
 		c->name = name;
 		c->type = MOD_GROUP;
@@ -934,7 +1231,9 @@ void modcallable_free(modcallable **pc)
 	modcallable *c, *loop, *next;
 	c = *pc;
 	if(c->type==MOD_GROUP) {
-		for(loop=mod_callabletogroup(c)->children ; loop ; loop=next) {
+		for(loop = mod_callabletogroup(c)->children;
+		    loop ;
+		    loop = next) {
 			next = loop->next;
 			modcallable_free(&loop);
 		}
