@@ -1,7 +1,7 @@
 /*
  * radiusd.c	Main loop of the radius server.
  *
- * Version:	$Id: radiusd.c,v 1.321.2.2 2005/04/11 23:45:21 aland Exp $
+ * Version:	$Id: radiusd.c,v 1.321.2.2.2.2 2005/12/28 01:18:00 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
 
 /* don't look here for the version, run radiusd -v or look in version.c */
 static const char rcsid[] =
-"$Id: radiusd.c,v 1.321.2.2 2005/04/11 23:45:21 aland Exp $";
+"$Id: radiusd.c,v 1.321.2.2.2.2 2005/12/28 01:18:00 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -1223,7 +1223,7 @@ int main(int argc, char *argv[])
 			 *	from under them.
 			 */
 		        int max_wait = 0;
-		        if (!spawn_flag) for(;;) {
+		        if (spawn_flag) for(;;) {
 			        /*
 				 * Block until there are '0' threads
 				 * with a REQUEST handle.
@@ -1890,7 +1890,7 @@ postpone_request:
 /*
  *  Display the syntax for starting this program.
  */
-static void usage(int status)
+static void NEVER_RETURNS usage(int status)
 {
 	FILE *output = status?stderr:stdout;
 
