@@ -26,10 +26,9 @@
  *
  */
 char radrelay_rcsid[] =
-"$Id: radrelay.c,v 1.22.2.3 2005/06/07 18:57:40 aland Exp $";
+"$Id: radrelay.c,v 1.22.2.3.2.1 2005/12/13 18:14:51 nbk Exp $";
 
 #include "autoconf.h"
-#include "libradius.h"
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -85,7 +84,9 @@ struct main_config_t mainconfig;
 #define		STATE_SHUTDOWN	3
 #define		STATE_CLOSE	4
 
-#define		NR_SLOTS	64
+#define		NR_SLOTS		64
+#define		DEFAULT_SLEEP		50
+#define		DEFAULT_SLEEP_EVERY	1
 
 /*
  *	A relay request.
@@ -107,6 +108,16 @@ struct relay_misc {
 	char		detail[1024];			/* Detail file */
 	char 		*secret;			/* Secret */
 	char		f_secret[256];			/* File secret */
+	int		sleep_time;			/* Time to sleep between sending packets */
+	int		sleep_every;			/* Sleep every so many packets */
+	int		records_print;			/* Print statistics after so many records */
+};
+
+struct relay_stats {
+	time_t		startup;
+	uint32_t	records_read;			/* Records read */
+	uint32_t	packets_sent;			/* Packets sent */
+	uint32_t	last_print_records;		/* Records on last statistics printout */
 };
 
 /*
@@ -169,7 +180,7 @@ void sigterm_handler(int sig)
 /*
  *	Sleep a number of milli seconds
  */
-void ms_sleep(int msec)
+inline void ms_sleep(int msec)
 {
 	struct timeval tv;
 
@@ -181,7 +192,7 @@ void ms_sleep(int msec)
 /*
  *	Does this (remotely) look like "Tue Jan 23 06:55:48 2001" ?
  */
-int isdateline(char *d)
+inline int isdateline(char *d)
 {
 	int y;
 
@@ -545,9 +556,10 @@ void loop(struct relay_misc *r_args)
 	FILE *fp = NULL;
 	struct relay_request *r;
 	struct timeval tv;
+	struct relay_stats stats;
 	fd_set readfds;
 	char work[1030];
-	time_t now, last_rename = 0;
+	time_t now, uptime, last_rename = 0;
 	int i, n;
 	int state = STATE_RUN;
 	int id;
@@ -558,6 +570,9 @@ void loop(struct relay_misc *r_args)
 
 	id = ((int)getpid() & 0xff);
 
+	memset(&stats,0,sizeof(struct relay_stats));
+	stats.startup = time(NULL);
+
 	/*
 	 * Initialize all our slots, might as well do this right away.
 	 */
@@ -655,6 +670,22 @@ void loop(struct relay_misc *r_args)
 				rad_unlockfd(fileno(fp), 0);
 				fseek(fp, fpos, SEEK_SET);
 			} while(0);
+			if (r_args->records_print && state == STATE_RUN){
+				stats.records_read++;
+				if (stats.last_print_records - stats.records_read >= r_args->records_print){
+					now = time(NULL);
+					uptime = (stats.startup == now) ? 1 : now - stats.startup;
+					fprintf(stderr, "%s: Running and Processing Records.\n",progname);
+					fprintf(stderr, "Seconds since startup: %ld\n",uptime);
+					fprintf(stderr, "Records Read: %d\n",stats.records_read);
+					fprintf(stderr, "Packets Sent: %d\n",stats.packets_sent);
+					fprintf(stderr, "Record Rate since startup: %.2f\n",
+						(double)stats.records_read / uptime);
+					fprintf(stderr, "Packet Rate since startup: %.2f\n",
+						(double)stats.packets_sent / uptime);
+					stats.last_print_records = stats.records_read;
+				}
+			}
 			if (r->state == STATE_FULL)
 				request_head = (request_head + 1) % NR_SLOTS;
 		}
@@ -705,11 +736,14 @@ void loop(struct relay_misc *r_args)
 		for (i = 0; i < NR_SLOTS; i++) {
 			if (slots[i].state == STATE_FULL) {
 				n += do_send(&slots[i], r_args->secret);
-				ms_sleep(140);
+				if ((n % r_args->sleep_every) == 0)
+					ms_sleep(r_args->sleep_time);
 				if (n > NR_SLOTS / 2)
 					break;
 			}
 		}
+		if (r_args->records_print)
+			stats.packets_sent += n;
 	}
 }
 
@@ -720,15 +754,13 @@ void loop(struct relay_misc *r_args)
 int find_shortname(char *shortname, char **host, char **secret)
 {
 	CONF_SECTION *maincs, *cs;
+	char buffer[256];
 
-	/*
-	 *	Ensure that the configuration is initialized.
-	 */
-	memset(&mainconfig, 0, sizeof(mainconfig));
-
-	if ((maincs = read_radius_conf_file()) == NULL) {
-		fprintf(stderr, "Error reading radiusd.conf\n");
-		exit(1);
+	/* Lets go look for the new configuration files */
+	memset(&mainconfig, 0, sizeof(mainconfig)); /* for radlog() */
+	snprintf(buffer, sizeof(buffer), "%.200s/radiusd.conf", radius_dir);
+	if ((maincs = conf_read(NULL, 0, buffer, NULL)) == NULL) {
+		return -1;
 	}
 
 	/*
@@ -743,8 +775,6 @@ int find_shortname(char *shortname, char **host, char **secret)
 		 * or we find one that matches.
 		 */
 		while (cs && strcmp(shortname, c_shortname)) {
-			free(c_shortname);
-			free(c_secret);
 			cs = cf_subsection_find_next(cs, cs, "client");
 			if (cs) {
 				c_shortname = cf_section_value_find(cs, "shortname");
@@ -766,7 +796,8 @@ int find_shortname(char *shortname, char **host, char **secret)
 void usage(void)
 {
 	fprintf(stderr, "Usage: radrelay [-a accounting_dir] [-d radius_dir] [-i local_ip] [-s secret]\n");
-	fprintf(stderr, "[-S secret_file] [-fx] <[-n shortname] [-r remote-server[:port]]> detailfile\n");
+	fprintf(stderr, "[-e sleep_every packets] [-t sleep_time (ms)] [-S secret_file] [-fx]\n");
+	fprintf(stderr, "[-R records_print] <[-n shortname] [-r remote-server[:port]]> detailfile\n");
 	fprintf(stderr, " -a accounting_dir     Base accounting directory.\n");
 	fprintf(stderr, " -d radius_dir         Base radius (raddb) directory.\n");
 	fprintf(stderr, " -f                    Stay in the foreground (don't fork).\n");
@@ -774,6 +805,11 @@ void usage(void)
 	fprintf(stderr, " -i local_ip           Use local_ip as source address.\n");
 	fprintf(stderr, " -n shortname          Use the [shortname] entry from clients.conf for\n");
 	fprintf(stderr, "                       ip-adress and secret.\n");
+	fprintf(stderr, " -t sleep_time		Sleep so much time (in ms) between sending packets. Default: %dms.\n",
+						DEFAULT_SLEEP);
+	fprintf(stderr, " -e sleep_every	Sleep after sending so many packets. Default: %d\n",
+						DEFAULT_SLEEP_EVERY);
+	fprintf(stderr, " -R records_print	If in foreground mode, print statistics after so many records read.\n");
 	fprintf(stderr, " -r remote-server      The destination address/hostname.\n");
 	fprintf(stderr, " -s secret             Server secret.\n");
 	fprintf(stderr, " -S secret_file        Read server secret from file.\n");
@@ -803,6 +839,8 @@ int main(int argc, char **argv)
 	memset((char *) r_args.detail, 0, 1024);
 	memset((char *) r_args.f_secret, 0, 256);
 	r_args.secret = NULL;
+	r_args.sleep_time = DEFAULT_SLEEP;
+	r_args.sleep_every = DEFAULT_SLEEP_EVERY;
 
 	shortname = NULL;
 	server_name = NULL;
@@ -820,7 +858,7 @@ int main(int argc, char **argv)
 	/*
 	 *	Process the options.
 	 */
-	while ((c = getopt(argc, argv, "a:d:fhi:n:r:s:S:x")) != EOF) switch(c) {
+	while ((c = getopt(argc, argv, "a:d:fhi:t:e:n:r:R:s:S:x")) != EOF) switch(c) {
 		case 'a':
 			if (strlen(optarg) > 1021) {
 				fprintf(stderr, "%s: acct_dir to long\n", progname);
@@ -839,6 +877,19 @@ int main(int argc, char **argv)
 		case 'n':
 			shortname = optarg;
 			break;
+		case 't':
+			r_args.sleep_time = atoi(optarg);
+			break;
+		case 'e':
+			r_args.sleep_every = atoi(optarg);
+			break;
+		case 'R':
+			if (!dontfork){
+				fprintf(stderr, "%s: Not in foreground mode. Can't print statistics.\n",progname);
+				usage();
+			}
+			r_args.records_print = atoi(optarg);
+			break;
 		case 'r':
 			server_name = optarg;
 			break;
