@@ -1,7 +1,7 @@
 /*
  * radeapclient.c	EAP specific radius packet debug tool.
  *
- * Version:	$Id: radeapclient.c,v 1.7 2004/02/26 19:04:29 aland Exp $
+ * Version:	$Id: radeapclient.c,v 1.7.4.1 2005/12/14 18:32:02 nbk Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  * Copyright 2000  Miquel van Smoorenburg <miquels@cistron.nl>
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
-static const char rcsid[] = "$Id: radeapclient.c,v 1.7 2004/02/26 19:04:29 aland Exp $";
+static const char rcsid[] = "$Id: radeapclient.c,v 1.7.4.1 2005/12/14 18:32:02 nbk Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -82,7 +82,7 @@ char password[256];
 
 struct eapsim_keys eapsim_mk;
 
-static void usage(void)
+static void NEVER_RETURNS usage(void)
 {
 	fprintf(stderr, "Usage: radeapclient [options] server[:port] <command> [<secret>]\n");
 
@@ -950,7 +950,7 @@ int main(int argc, char **argv)
 			timeout = atof(optarg);
 			break;
 		case 'v':
-			printf("radclient: $Id: radeapclient.c,v 1.7 2004/02/26 19:04:29 aland Exp $ built on " __DATE__ " at " __TIME__ "\n");
+			printf("radclient: $Id: radeapclient.c,v 1.7.4.1 2005/12/14 18:32:02 nbk Exp $ built on " __DATE__ " at " __TIME__ "\n");
 			exit(0);
 			break;
                case 'S':
