@@ -1,7 +1,7 @@
 /*
  * peap.c  contains the interfaces that are called from eap
  *
- * Version:     $Id: peap.c,v 1.11 2004/03/12 18:23:14 aland Exp $
+ * Version:     $Id: peap.c,v 1.11.4.1 2005/12/08 13:24:32 nbk Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -393,7 +393,7 @@ static int eappeap_postproxy(EAP_HANDLER *handler, void *data)
 		fake->options &= ~RAD_REQUEST_OPTION_PROXY_EAP;
 		DEBUG2("  PEAP: Passing reply back for EAP-MS-CHAP-V2 %p %d",
 		       fake, fake->reply->code);
-		rcode = module_post_proxy(fake);
+		rcode = module_post_proxy(0, fake);
 
 		/*
 		 *	FIXME: If rcode returns fail, do something
