@@ -157,7 +157,7 @@
  *	  a corresponding ldap_groupcmp function
  *	- Small change to ldap_get_conn to fix problems on some platforms
  */
-static const char rcsid[] = "$Id: rlm_ldap.c,v 1.122.2.7 2005/08/29 16:09:56 aland Exp $";
+static const char rcsid[] = "$Id: rlm_ldap.c,v 1.122.2.8.2.2 2005/12/29 21:52:53 aland Exp $";
 
 #include "autoconf.h"
 
@@ -309,6 +309,7 @@ typedef struct {
 #ifdef NOVELL
 	int			edir_account_policy_check;
 #endif
+	int		set_auth_type;
 }               ldap_instance;
 
 /* The default setting for TLS Certificate Verification */
@@ -356,6 +357,7 @@ static CONF_PARSER module_config[] = {
 	{"edir_account_policy_check", PW_TYPE_BOOLEAN, offsetof(ldap_instance,edir_account_policy_check), NULL, "yes"},
 #endif
 
+	{"set_auth_type", PW_TYPE_BOOLEAN, offsetof(ldap_instance,set_auth_type), NULL, "yes"},
 	{NULL, -1, 0, NULL, NULL}
 };
 
@@ -417,6 +419,7 @@ ldap_instantiate(CONF_SECTION * conf, void **instance)
 	TLDAP_RADIUS *pair;
 	ATTR_FLAGS flags;
 	char *xlat_name;
+	DICT_VALUE *dv;
 
 	inst = rad_malloc(sizeof *inst);
 	if (!inst) {
@@ -491,6 +494,19 @@ ldap_instantiate(CONF_SECTION * conf, void **instance)
 	DEBUG("rlm_ldap: Registering ldap_xlat with xlat_name %s",xlat_name);
 	xlat_register(xlat_name,ldap_xlat,inst);
 
+	/*
+	 *	Over-ride set_auth_type if there's no Auth-Type of our name.
+	 *	This automagically catches the case where LDAP is listed
+	 *	in "authorize", but not "authenticate".
+	 */
+	dv = dict_valbyname(PW_AUTH_TYPE, xlat_name);
+	if (!dv) {
+		if (inst->set_auth_type) {
+			DEBUG2("rlm_ldap: Over-riding set_auth_type, as we're not listed in the \"authenticate\" section.");
+		}
+	  inst->set_auth_type = 0;
+	}
+
 #ifdef NOVELL
 	/*
 	 * (LDAP_Instance, V1) attribute-value pair in the config items list means
@@ -828,25 +844,36 @@ static int ldap_escape_func(char *out, int outlen, const char *in)
 
 	while (in[0]) {
 		/*
-		 *  Only one byte left.
+		 *	Encode unsafe characters.
 		 */
-		if (outlen <= 1) {
-			break;
-		}
-
 		if (strchr("*=\\,()", *in)) {
-			static const char *hex = "0123456789abcdef";
-			if (outlen <= 3) break;
+			static const char hex[] = "0123456789abcdef";
+
+			/*
+			 *	Only 3 or less bytes available.
+			 */
+			if (outlen <= 3) {
+				break;
+			}
 
 			*(out++) = '\\';
 			*(out++) = hex[((*in) >> 4) & 0x0f];
 			*(out++) = hex[(*in) & 0x0f];
 			outlen -= 3;
+			len += 3;
+			in++;
 			continue;
 		}
 
 		/*
-		 *	Else it's a nice character.
+		 *	Only one byte left.
+		 */
+		if (outlen <= 1) {
+			break;
+		}
+
+		/*
+		 *	Allowed character.
 		 */
 		*(out++) = *(in++);
 		outlen--;
@@ -1586,13 +1613,15 @@ ldap_authorize(void *instance, REQUEST * request)
 
 	/*
  	 * Module should default to LDAP authentication if no Auth-Type
-	 * specified
+	 * specified.  Note that we do this ONLY if configured, AND we
+	 * set the Auth-Type to our module name, which allows multiple
+	 * ldap instances to work.
 	 */
-	if ((pairfind(*check_pairs, PW_AUTH_TYPE) == NULL) &&
+	if (inst->set_auth_type &&
+	    (pairfind(*check_pairs, PW_AUTH_TYPE) == NULL) &&
 	    request->password &&
 	    (request->password->attribute == PW_USER_PASSWORD))
-		pairadd(check_pairs, pairmake("Auth-Type", "LDAP", T_OP_EQ));
-
+		pairadd(check_pairs, pairmake("Auth-Type", inst->xlat_name, T_OP_EQ));
 
 	DEBUG("rlm_ldap: user %s authorized to use remote access",
 	      request->username->strvalue);
@@ -1972,7 +2001,7 @@ ldap_connect(void *instance, const char *dn, const char *password, int auth, int
 	}
 
 	if(inst->tls_cacertdir != NULL) {
-		DEBUG("rlm_ldap: setting TLS CACert File to %s", inst->tls_cacertdir);
+		DEBUG("rlm_ldap: setting TLS CACert Directory to %s", inst->tls_cacertdir);
 
 		if ( ldap_set_option( NULL, LDAP_OPT_X_TLS_CACERTDIR,
 							  (void *) inst->tls_cacertdir )
