@@ -2,7 +2,7 @@
  * rlm_sql.c		SQL Module
  * 		Main SQL module file. Most ICRADIUS code is located in sql.c
  *
- * Version:	$Id: rlm_sql.c,v 1.131.2.4 2005/08/31 12:49:52 nbk Exp $
+ * Version:	$Id: rlm_sql.c,v 1.131.2.8.2.1 2005/12/13 14:11:53 nbk Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
  */
 
 static const char rcsid[] =
-	"$Id: rlm_sql.c,v 1.131.2.4 2005/08/31 12:49:52 nbk Exp $";
+	"$Id: rlm_sql.c,v 1.131.2.8.2.1 2005/12/13 14:11:53 nbk Exp $";
 
 #include "autoconf.h"
 
@@ -620,6 +620,7 @@ static int rlm_sql_detach(void *instance)
 			free(*p);
 			*p = NULL;
 		}
+		allowed_chars = NULL;
 		free(inst->config);
 		inst->config = NULL;
 	}
@@ -755,14 +756,8 @@ static int rlm_sql_authorize(void *instance, REQUEST * request)
 		return RLM_MODULE_INVALID;
 	}
 
-
 	/*
-	 *  After this point, ALL 'return's MUST release the SQL socket!
-	 */
-
-
-	/*
-	 * Set, escape, and check the user attr here
+	 *	Set, escape, and check the user attr here.
 	 */
 	if (sql_set_user(inst, request, sqlusername, NULL) < 0)
 		return RLM_MODULE_FAIL;
@@ -772,9 +767,13 @@ static int rlm_sql_authorize(void *instance, REQUEST * request)
 	if (sqlsocket == NULL) {
 		/* Remove the username we (maybe) added above */
 		pairdelete(&request->packet->vps, PW_SQL_USER_NAME);
-		return(RLM_MODULE_FAIL);
+		return RLM_MODULE_FAIL;
 	}
 
+	/*
+	 *	After this point, ALL 'return's MUST release the SQL socket!
+	 */
+
 	found = sql_getvpdata(inst, sqlsocket, &check_tmp, querystr, PW_VP_USERDATA);
 	/*
 	 *      Find the entry for the user.
@@ -792,6 +791,7 @@ static int rlm_sql_authorize(void *instance, REQUEST * request)
 		sql_release_socket(inst, sqlsocket);
 		/* Remove the username we (maybe) added above */
 		pairdelete(&request->packet->vps, PW_SQL_USER_NAME);
+		pairfree(&check_tmp);
 		return RLM_MODULE_FAIL;
 
 	} else {
@@ -826,6 +826,9 @@ static int rlm_sql_authorize(void *instance, REQUEST * request)
 				radlog(L_DBG, "rlm_sql (%s): Checking profile %s",
 				       inst->config->xlat_name, profile);
 				if (sql_set_user(inst, request, sqlusername, profile) < 0) {
+					sql_release_socket(inst, sqlsocket);
+					pairfree(&reply_tmp);
+					pairfree(&check_tmp);
 					return RLM_MODULE_FAIL;
 				}
 				radius_xlat(querystr, sizeof(querystr), inst->config->authorize_group_check_query,
@@ -839,12 +842,19 @@ static int rlm_sql_authorize(void *instance, REQUEST * request)
 			}
 		}
 	}
+
+	/*
+	 *	We don't need the SQL socket anymore.
+	 */
+	sql_release_socket(inst, sqlsocket);
+
 	if (!found) {
 		radlog(L_DBG, "rlm_sql (%s): User not found",
 		       inst->config->xlat_name);
-		sql_release_socket(inst, sqlsocket);
 		/* Remove the username we (maybe) added above */
 		pairdelete(&request->packet->vps, PW_SQL_USER_NAME);
+		pairfree(&reply_tmp);
+		pairfree(&check_tmp);
 		return RLM_MODULE_NOTFOUND;
 	}
 
@@ -865,7 +875,6 @@ static int rlm_sql_authorize(void *instance, REQUEST * request)
 		       inst->config->xlat_name, sqlusername);
 		/* Remove the username we (maybe) added above */
 		pairdelete(&request->packet->vps, PW_SQL_USER_NAME);
-		sql_release_socket(inst, sqlsocket);
 		pairfree(&reply_tmp);
 		pairfree(&check_tmp);
 		return RLM_MODULE_NOTFOUND;
@@ -878,7 +887,6 @@ static int rlm_sql_authorize(void *instance, REQUEST * request)
 
 	/* Remove the username we (maybe) added above */
 	pairdelete(&request->packet->vps, PW_SQL_USER_NAME);
-	sql_release_socket(inst, sqlsocket);
 
 	return RLM_MODULE_OK;
 }
@@ -910,8 +918,9 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 	if ((pair = pairfind(request->packet->vps, PW_ACCT_STATUS_TYPE)) != NULL) {
 		acctstatustype = pair->lvalue;
 	} else {
-		radius_xlat(logstr, sizeof(logstr), "rlm_sql: packet has no account status type.  [user '%{User-Name}', nas '%{NAS-IP-Address}']", request, sql_escape_func);
-		radlog(L_ERR, logstr);
+		radius_xlat(logstr, sizeof(logstr), "packet has no accounting status type. [user '%{User-Name}', nas '%{NAS-IP-Address}']", request, NULL);
+		radlog(L_ERR, "rlm_sql (%s) in sql_accounting: %s",
+		       inst->config->xlat_name, logstr);
 		return RLM_MODULE_INVALID;
 	}
 
@@ -1077,12 +1086,12 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 					        if ((pair = pairfind(request->packet->vps, PW_ACCT_SESSION_TIME)) != NULL)
 					                acctsessiontime = pair->lvalue;
 
-					        if (acctsessiontime <= 0) {
-				        	        radius_xlat(logstr, sizeof(logstr), "rlm_sql: Stop packet with zero session length.  (user '%{User-Name}', nas '%{NAS-IP-Address}')", request, sql_escape_func);
-					                radlog(L_ERR, logstr);
-				        	        sql_release_socket(inst, sqlsocket);
-				                	ret = RLM_MODULE_NOOP;
-					        }
+						if (acctsessiontime <= 0) {
+							radius_xlat(logstr, sizeof(logstr), "stop packet with zero session length. [user '%{User-Name}', nas '%{NAS-IP-Address}']", request, NULL);
+							radlog(L_ERR, "rlm_sql (%s) in sql_accounting: %s", inst->config->xlat_name, logstr);
+							sql_release_socket(inst, sqlsocket);
+							ret = RLM_MODULE_NOOP;
+						}
 #endif
 
 						radius_xlat(querystr, sizeof(querystr), inst->config->accounting_stop_query_alt, request, sql_escape_func);
