@@ -8,14 +8,10 @@
 #ifndef _RLM_SQL_H
 #define _RLM_SQL_H
 
-#if HAVE_PTHREAD_H
+#ifdef HAVE_PTHREAD_H
 #include        <pthread.h>
 #endif
 
-#if HAVE_SEMAPHORE_H
-#include        <semaphore.h>
-#endif
-
 #include	<ltdl.h>
 
 #include "conf.h"
@@ -35,10 +31,8 @@ typedef char** SQL_ROW;
 
 typedef struct sql_socket {
 	int     id;
-#if HAVE_SEMAPHORE_H
-	sem_t  *semaphore;
-#else
-	int     in_use;
+#ifdef HAVE_PTHREAD_H
+	pthread_mutex_t mutex;
 #endif
 	struct sql_socket *next;
 	enum { sockconnected, sockunconnected } state;
@@ -66,17 +60,13 @@ typedef struct rlm_sql_module_t {
 } rlm_sql_module_t;
 
 typedef struct sql_inst {
-	int		used;
-	time_t connect_after;
+	time_t		connect_after;
 	SQLSOCK		*sqlpool;
+	SQLSOCK		*last_used;
 	SQL_CONFIG	*config;
 
+	lt_dlhandle handle;
 	rlm_sql_module_t *module;
-
-        int socknr;
-#if HAVE_PTHREAD_H
-        pthread_mutex_t mutex;
-#endif
 } SQL_INST;
 
 
@@ -91,7 +81,7 @@ int     sql_getvpdata(SQL_INST * inst, SQLSOCK * sqlsocket, VALUE_PAIR **pair, c
 int     sql_read_naslist(SQLSOCK * sqlsocket);
 int     sql_read_clients(SQLSOCK * sqlsocket);
 int     sql_dict_init(SQLSOCK * sqlsocket);
-void    query_log(SQL_INST * inst, char *querystr);
+void    query_log(REQUEST *request, SQL_INST * inst, char *querystr);
 int	rlm_sql_select_query(SQLSOCK *sqlsocket, SQL_INST *inst, char *query);
 int	rlm_sql_query(SQLSOCK *sqlsocket, SQL_INST *inst, char *query);
 int	rlm_sql_fetch_row(SQLSOCK *sqlsocket, SQL_INST *inst);
