@@ -1,7 +1,7 @@
 /*
  * rlm_sqlcounter.c
  *
- * Version:  $Id: rlm_sqlcounter.c,v 1.11.2.3 2005/08/23 14:13:38 nbk Exp $
+ * Version:  $Id: rlm_sqlcounter.c,v 1.11.2.3.2.4 2005/12/27 17:44:25 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -56,7 +56,7 @@
  */
 
 
-static const char rcsid[] = "$Id: rlm_sqlcounter.c,v 1.11.2.3 2005/08/23 14:13:38 nbk Exp $";
+static const char rcsid[] = "$Id: rlm_sqlcounter.c,v 1.11.2.3.2.4 2005/12/27 17:44:25 aland Exp $";
 
 /*
  *	Define a structure for our module configuration.
@@ -72,6 +72,7 @@ typedef struct rlm_sqlcounter_t {
 	char *sqlmod_inst;	/* instance of SQL module to use, usually just 'sql' */
 	char *query;		/* SQL query to retrieve current session time */
 	char *reset;  		/* daily, weekly, monthly, never or user defined */
+	char *allowed_chars;	/* safe characters list for SQL queries */
 	time_t reset_time;
 	time_t last_reset;
 	int  key_attr;		/* attribute number for key field */
@@ -94,14 +95,11 @@ static CONF_PARSER module_config[] = {
   { "sqlmod-inst", PW_TYPE_STRING_PTR, offsetof(rlm_sqlcounter_t,sqlmod_inst), NULL, NULL },
   { "query", PW_TYPE_STRING_PTR, offsetof(rlm_sqlcounter_t,query), NULL, NULL },
   { "reset", PW_TYPE_STRING_PTR, offsetof(rlm_sqlcounter_t,reset), NULL,  NULL },
+  { "safe-characters", PW_TYPE_STRING_PTR, offsetof(rlm_sqlcounter_t,allowed_chars), NULL, "@abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.-_: /"},
   { NULL, -1, 0, NULL, NULL }
 };
 
-/*
- *	Safe characters list for sql queries. Everything else is
- *	replaced with their mime-encoded equivalents.
- */
-static const char allowed_chars[] = "@abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.-_: /";
+static char *allowed_chars = NULL;
 
 /*
  *	Translate the SQL queries.
@@ -428,6 +426,7 @@ static int sqlcounter_instantiate(CONF_SECTION *conf, void **instance)
 	DICT_ATTR *dattr;
 	ATTR_FLAGS flags;
 	time_t now;
+	char buffer[MAX_STRING_LEN];
 
 	/*
 	 *	Set up a storage area for instance data
@@ -447,6 +446,20 @@ static int sqlcounter_instantiate(CONF_SECTION *conf, void **instance)
 		return -1;
 	}
 
+	/*
+	 *	No query, die.
+	 */
+	if (data->query == NULL) {
+		radlog(L_ERR, "rlm_sqlcounter: 'query' must be set.");
+		return -1;
+	}
+
+	/*
+	 *	Safe characters list for sql queries. Everything else is
+	 *	replaced rwith their mime-encoded equivalents.
+	 */
+	allowed_chars = data->allowed_chars;
+
 	/*
 	 *	Discover the attribute number of the key.
 	 */
@@ -454,6 +467,11 @@ static int sqlcounter_instantiate(CONF_SECTION *conf, void **instance)
 		radlog(L_ERR, "rlm_sqlcounter: 'key' must be set.");
 		return -1;
 	}
+	sql_escape_func(buffer, sizeof(buffer), data->key_name);
+	if (strcmp(buffer, data->key_name) != 0) {
+		radlog(L_ERR, "rlm_sqlcounter: The value for option 'key' is too long or contains unsafe characters.");
+		return -1;
+	}
 	dattr = dict_attrbyname(data->key_name);
 	if (dattr == NULL) {
 		radlog(L_ERR, "rlm_sqlcounter: No such attribute %s",
@@ -462,6 +480,18 @@ static int sqlcounter_instantiate(CONF_SECTION *conf, void **instance)
 	}
 	data->key_attr = dattr->attr;
 
+	/*
+	 *	Check the "sqlmod-inst" option.
+	 */
+	if (data->sqlmod_inst == NULL) {
+		radlog(L_ERR, "rlm_sqlcounter: 'sqlmod-inst' must be set.");
+		return -1;
+	}
+	sql_escape_func(buffer, sizeof(buffer), data->sqlmod_inst);
+	if (strcmp(buffer, data->sqlmod_inst) != 0) {
+		radlog(L_ERR, "rlm_sqlcounter: The value for option 'sqlmod-inst' is too long or contains unsafe characters.");
+		return -1;
+	}
 
 	/*
 	 *  Create a new attribute for the counter.
@@ -692,6 +722,8 @@ static int sqlcounter_detach(void *instance)
 	free(data->check_name);
 	free(data->sqlmod_inst);
 	free(data->counter_name);
+	free(data->allowed_chars);
+	allowed_chars = NULL;
 
 	free(instance);
 	return 0;
